from setuptools import setup

setup(
    name="razorpay",
    version="0.1.0",
    description="Razorpay Python Client",
    url="https://github.com/decached/razorpay",
    author="Akash Kothawale",
    author_email="akash@decached.com",
    license="MIT",
    install_requires=["requests"],
    packages=["razorpay"],
    keywords='razorpay payment gateway india',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)
